{-# OPTIONS -#include "HsStd.h" #-}
{-# LINE 1 "CPUTime.hsc" #-}
-- -----------------------------------------------------------------------------
{-# LINE 2 "CPUTime.hsc" #-}
-- $Id: CPUTime.hsc,v 1.10.4.2 2001/09/06 16:41:35 sewardj Exp $
--
-- (c) The University of Glasgow, 1995-2001
--

module CPUTime 
	(
         getCPUTime,       -- :: IO Integer
	 cpuTimePrecision  -- :: Integer
        ) where

import PrelMarshalAlloc
import PrelMarshalUtils ( toBool )
import PrelCTypesISO
import PrelCTypes
import PrelStorable
import PrelPtr

import PrelBase		( Int(..) )
import PrelByteArr  	( ByteArray(..), newIntArray )
import PrelArrExtra     ( unsafeFreezeByteArray )
import PrelIOBase	( IOException(..) )
import Ratio


{-# LINE 27 "CPUTime.hsc" #-}

-- -----------------------------------------------------------------------------
-- Computation `getCPUTime' returns the number of picoseconds CPU time
-- used by the current program.  The precision of this result is
-- implementation-dependent.

-- The `cpuTimePrecision' constant is the smallest measurable difference
-- in CPU time that the implementation can record, and is given as an
-- integral number of picoseconds.

getCPUTime :: IO Integer
getCPUTime = do


{-# LINE 41 "CPUTime.hsc" #-}
-- getrusage() is right royal pain to deal with when targetting multiple
-- versions of Solaris, since some versions supply it in libc (2.3 and 2.5),
-- while 2.4 has got it in libucb (I wouldn't be too surprised if it was back
-- again in libucb in 2.6..)
--
-- Avoid the problem by resorting to times() instead.
--

{-# LINE 49 "CPUTime.hsc" #-}
    allocaBytes (72) $ \ p_rusage -> do
{-# LINE 50 "CPUTime.hsc" #-}
    getrusage (0) p_rusage
{-# LINE 51 "CPUTime.hsc" #-}

    let ru_utime = ((\hsc_ptr -> hsc_ptr `plusPtr` 0)) p_rusage
{-# LINE 53 "CPUTime.hsc" #-}
    let ru_stime = ((\hsc_ptr -> hsc_ptr `plusPtr` 8)) p_rusage
{-# LINE 54 "CPUTime.hsc" #-}
    u_sec  <- ((\hsc_ptr -> peekByteOff hsc_ptr 0))  ru_utime :: IO CTime
{-# LINE 55 "CPUTime.hsc" #-}
    u_usec <- ((\hsc_ptr -> peekByteOff hsc_ptr 4)) ru_utime :: IO CTime
{-# LINE 56 "CPUTime.hsc" #-}
    s_sec  <- ((\hsc_ptr -> peekByteOff hsc_ptr 0))  ru_stime :: IO CTime
{-# LINE 57 "CPUTime.hsc" #-}
    s_usec <- ((\hsc_ptr -> peekByteOff hsc_ptr 4)) ru_stime :: IO CTime
{-# LINE 58 "CPUTime.hsc" #-}

    return ((fromIntegral u_sec * 1000000 + fromIntegral u_usec + 
             fromIntegral s_sec * 1000000 + fromIntegral s_usec) 
		* 1000000)

type CRUsage = ()
foreign import unsafe getrusage :: CInt -> Ptr CRUsage -> IO CInt

{-# LINE 83 "CPUTime.hsc" #-}


{-# LINE 111 "CPUTime.hsc" #-}

cpuTimePrecision :: Integer
cpuTimePrecision = round ((1000000000000::Integer) % fromIntegral (clockTicks))

clockTicks :: Int
clockTicks =

{-# LINE 118 "CPUTime.hsc" #-}
    (100)
{-# LINE 119 "CPUTime.hsc" #-}

{-# LINE 123 "CPUTime.hsc" #-}
